call sysproc.set_routine_opts('ISOLATION UR') @

drop specific procedure exec_sql_and_log @
drop specific procedure exec_sqlparm_and_log @
drop specific procedure roll_partition    @
drop specific procedure truncate    @


CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,
                                    IN i_dynstmt          VARCHAR(1024),
                                    OUT o_sqlcode         INTEGER )

SPECIFIC exec_sql_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/08/31   jicimi   created       
--      2007/09/20   jicimi   added get_diagnostic call      
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 times
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(30000) DEFAULT '';  
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0;
      SET v_sqlmsg = '';
      IF ( i_preview_only_ind = '0' ) THEN
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
            select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        --SIGNAL SQLSTATE '99001';
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE IMMEDIATE i_dynStmt;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
        END ANONYMOUS_BLOCK;
      END IF;
    
    
      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt, i_preview_only_ind, v_rows,  v_sqlcode, v_sqlstate,v_sqlmsg );
      --40001 is deadlock/timeout
      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN

          SET v_attempts = i_locktimeout_cnt+1 ;

        ELSE
          SET v_attempts = v_attempts + 1 ;         
		  call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;

END P1
@


CREATE PROCEDURE EXEC_SQL_AND_LOG ( IN i_tabname          VARCHAR(50),
                                    IN i_schema           VARCHAR(30),
                                    IN i_partition        VARCHAR(50),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt  INTEGER,                                    
                                    IN i_dynstmt          VARCHAR(1024),
                                    IN i_parameter        VARCHAR(1024),
                                    OUT o_sqlcode         INTEGER )

SPECIFIC exec_sqlparm_and_log
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  
--
--
-- Description:
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2007/10/31   jicimi   added loop to trap -911s and 
--                            retry upto 3 time
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            the retry count is not passed in, so its configurable 
--      2011/11/30   leushe   Added support to retry for sqlstates 55057 and 20285 for db2 9.7 Linux AIC contention
--      2012/04/26   leushe   instead of looping for 30secs, I added the call to the Sleep Proc     
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0 ;
  DECLARE v_sqlstate char(5) DEFAULT '00000';
  DECLARE v_sqlmsg VARCHAR(30000) DEFAULT '';
  DECLARE v_rows integer DEFAULT 0 ;
  DECLARE v_attempts integer DEFAULT 1 ;
  DECLARE v_stmt STATEMENT;
  DECLARE s1 STATEMENT;


  WHILE  ( v_attempts <= i_locktimeout_cnt ) DO

      SET v_sqlstate = '00000';
      SET v_sqlcode = 0 ;
      SET v_sqlmsg = '';
      
      IF ( i_preview_only_ind = '0' ) THEN
    
        ANONYMOUS_BLOCK: BEGIN
        DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
        BEGIN
           get diagnostics exception 1 v_sqlmsg = message_text;
           select sqlcode, sqlstate into v_sqlcode, v_sqlstate from sysibm.sysdummy1 ;
         END;
        PREPARE v_stmt FROM i_dynStmt;
        EXECUTE v_stmt USING i_parameter ;
        GET DIAGNOSTICS v_rows = ROW_COUNT;
    
        END ANONYMOUS_BLOCK;
      END IF;
    

      CALL audit_partition_action  (i_tabname, i_schema, i_partition, 
                        i_dynStmt||' ?='||i_parameter, i_preview_only_ind, v_rows,
                        v_sqlcode, v_sqlstate,v_sqlmsg );

      IF ( (v_sqlstate != '40001') AND
           (v_sqlstate != '55057') AND 
           (posstr(v_sqlmsg,'20285') = 0) ) THEN
          SET v_attempts = i_locktimeout_cnt+1 ;
      ELSE
          SET v_attempts = v_attempts + 1 ;
          call sleep(30);
      END IF;

  END WHILE;

  SET o_sqlcode = v_sqlcode ;


END P1
@

CREATE PROCEDURE truncate         ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    IN i_detach_wait_sec INTEGER,
                                    OUT o_sqlcode INT  )
SPECIFIC truncate         
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  truncate
--
--
-- Description:
--     Given the name of a table truncate it by invoking the loader.
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT 
--                            to pass to exec_sql_and_log procedure 
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--            
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_dynStmtParm     VARCHAR(1024);
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer;
  DECLARE v_sqlstate char(5) ;
  
  DECLARE v_stmt STATEMENT;


  --
  -- build the dynamic truncate statement via import /dev/null
  --
  SET v_dynStmtParm = 'IMPORT FROM /dev/null of del replace into '||i_schema||'.'||i_tabname;

  --
  -- create the dynamic sql statement
  -- 
  SET v_dynStmt = 'call sysproc.admin_cmd (?)';
  
  set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and length(trim(status)) >0 
                  );
  END WHILE;

  CALL exec_sql_and_log 
        (i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt,v_dynStmt, v_dynStmtParm, o_sqlcode) ;


END P1
@

CREATE PROCEDURE roll_partition ( IN i_tabname          VARCHAR(50),
                                     IN i_schema           VARCHAR(30)
                                       )
SPECIFIC ROLL_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ROLL_PARTITION
--
--
-- Description:
--     Top level entry point for the RH partitioning maintenance.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/26   jicimi   created       
--      2007/11/05   jicimi   added insert into frx_partition_definition
--                            to control how often the code is allowed to  
--                            run - sanity_check queries this column
--      2009/08/19   leushe   renamed the procedure to roll_partition
--							  changed the call to attach_rh_partition to the procedures
--                            new name ATTACH_PARTITION
--      2009/08/24   leushe   Changed the proc to not return explicit parameters, but return 
--                            the sqlcode via a result set. this changed the signature of the proc
--                            This change was made so SAS could call the proc, as it doesn't handle explicit
--                            Output parameters
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT Column on the partition_definition table 
--      2012/04/26   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the update_cf_partitions proc
--    
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_rowCount        INTEGER   DEFAULT 0;
  DECLARE v_detached_table  VARCHAR(50);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0;
  DECLARE v_sqlstate varchar(50) DEFAULT '00000' ;
  DECLARE v_sqlmsg varchar(30000)  ;
  DECLARE l_preview_only_ind CHAR(1) DEFAULT '0';
  DECLARE l_chk_sanity_ind CHAR(1) DEFAULT '0';
  DECLARE l_locktimeout_cnt INTEGER DEFAULT 3;
  DECLARE l_detach_wait_sec integer DEFAULT 30;
  
      /* Declare cursor to return result set to calling program */ 
        
        DECLARE c_result CURSOR WITH RETURN FOR
            SELECT   v_sqlcode as OUT_SQLCODE
                    ,v_sqlstate as OUT_SQLSTATE
            FROM    SYSIBM.SYSDUMMY1;
  
      /* Initialize variables */
        VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;

 
  -- 1.  upcase all arguments 
  SET i_tabname = upper(i_tabname);
  SET i_schema = upper(i_schema);
  SET v_sqlstate = i_tabname ;
 
  --
  -- verify i_tabname is indeed a partitioned table
  --
  SET v_rowCount = ( select count(*) from frx_partition_definition where table_name = i_tabname );

  IF ( v_rowCount != 1 ) THEN
    SET v_sqlcode = -1;
    SET v_sqlstate = '-1';
    SET v_sqlmsg = '';
    CALL audit_partition_action(i_tabname, i_schema, NULL, i_tabname||' is not partition eligible.',
                                '0', v_sqlcode, v_sqlstate,v_sqlmsg );
          
    SET v_sqlstate = i_tabname;                                
    OPEN c_result;

    RETURN -1;
  END IF;
 
  Select LOCKTIMEOUT_RETRY_COUNT, PREVIEW_ONLY_IND,SKIP_SANITY_CHECK_IND,DETACH_WAIT_SEC 
  		INTO l_locktimeout_cnt,l_preview_only_ind,l_chk_sanity_ind,l_detach_wait_sec
  FROM FRX_PARTITION_DEFINITION
  where table_name = i_tabname;
  
 
  IF ( l_chk_sanity_ind = '0' ) THEN
    -- 2. sanity check its ok to go on
    call sanity_check_partition(i_tabname, i_schema, l_preview_only_ind, v_sqlcode) ;

    IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
   END IF; 
  END IF;
 

  -- 3. attach the detached partiton as the new one
  call ATTACH_PARTITION(i_tabname, i_schema, l_preview_only_ind,l_locktimeout_cnt, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
  END IF;



  -- 4. detach oldest
  call detach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
  END IF;

   
  -- 5. empty oldest
  call truncate(v_detached_table, i_schema, l_preview_only_ind,l_locktimeout_cnt, l_detach_wait_sec,v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
  END IF;


  -- 6. insert current timestamp into definition table
  --    this value is used to ensure the code does not
  --    execute more frequently than it is supposed to
     IF ( l_preview_only_ind = '0' ) THEN
        update frx_partition_definition 
         set last_run_timestamp = ( current timestamp )
         where table_name = i_tabname ;
     END IF;
  
  VALUES ( SQLCODE) INTO  v_sqlcode;
  COMMIT;
  OPEN c_result;  
END P1
@


