
call sysproc.set_routine_opts('ISOLATION UR') @


CREATE PROCEDURE ATTACH_PARTITION ( IN i_tabname VARCHAR(50),
                                       IN i_schema  VARCHAR(30), 
                                       IN i_preview_only_ind CHAR(1),
                                       IN i_locktimeout_cnt INTEGER,                                       
                                       OUT o_sqlcode INT  )
SPECIFIC ATTACH_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ATTACH_PARTITION
--
--
-- Description:
--     This should be the normal path through the attach code for all RH
--     partitions.
--
--     Given the name of a partioned table & a free standing table
--     attach the free standing table as a new partition.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--           
--
-- Usage:
--
--
-- History:    
--      2007/09/26   jicimi   created       
--      2007/12/07   jicimi   corrected substr command at line 951
--      2009/07/15   leushe   change the upper bound to be the last day of the month
--                            when partitioing by MONTH
--      2009/08/19   leushe   renamed the procedure to ATTACH_PARTITION
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log procedure
--      2012/07/13   leushe   Fixed the upper bound calculation for Month Partitioning 
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name    VARCHAR(50);
  DECLARE v_partition          VARCHAR(50);
  DECLARE v_dynStmt            VARCHAR(1024);
  DECLARE v_dateStr            VARCHAR(30);
  DECLARE v_lowerBoundStr      VARCHAR(30);
  DECLARE v_upperBoundStr      VARCHAR(30);
  DECLARE v_newlowerBoundStr   VARCHAR(30);
  DECLARE v_newupperBoundStr   VARCHAR(30);
  DECLARE v_workStr            VARCHAR(30);
  DECLARE v_timestamp          TIMESTAMP;
  DECLARE uniqueStr            VARCHAR(10);
  DECLARE v_granularity        INTEGER;
  DECLARE v_granularity_unit   CHAR(10);


  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the detached table/partition name
  -- 
  SET v_table_part_name = 
      ( select tabname from syscat.tables a
        where tabname like i_tabname || '_DTCH_PART%' 
        and tabschema = i_schema
        order by create_time 
        fetch first row only
      );


  --
  -- to assign the new lowerbound 
  -- get the current upperbound of the last NON CF partition
  -- add 1 second to that value and that becomes the lowerbound of the new partition
  -- this essentially anchors the lowerbound so no gaps are introduced 
  -- if the partition definition table gets updated on the fly
  --
  -- to assign the new upperbound
  -- get the current upperbound of the last NON CF partition
  -- get the granularity value for the table from the partition definition table
  -- add the two togther and that becomes the new upperbound 
  -- 
  --

  --
  -- get the current upperbound of the last NON CF partition
  -- 
  SET v_lowerBoundStr =
      ( select cast(  max(highvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname
        and datapartitionname like 'PART%'
        and datapartitionname not like '%THE_END%'
        and tabschema = i_schema
      );


  --
  -- get the current upperbound of the last NON CF partition
  --
  SET v_upperBoundStr =
      ( select cast( max(highvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname
        and datapartitionname like 'PART%'
        and datapartitionname not like '%THE_END%'
        and tabschema = i_schema
      );



  --
  -- get the increment for the upper partition bound
  --
  SET v_granularity = 
      ( select granularity 
        from frx_partition_definition 
        where table_name = i_tabname 
      );


  --
  -- get the granularity unit ( HOUR, DAY, MONTH )  for the upper partition bound
  --
  SET v_granularity_unit = 
      ( select granularity_unit 
        from frx_partition_definition 
        where table_name = i_tabname 
      );


  --
  -- get the new upperBoundStr & lowerBoundStr values
  --    take the str representation from the catalog
  --    convert it to a date/timestamp
  --    the # of days to add should be driven by the metadata table
  --        add the granularity using date arihematic 
  --    then convert the timestamp back to a string for the attach call
  --

  SET v_upperBoundStr = STRIP(v_upperBoundStr, BOTH, '''');
  SET v_lowerBoundStr = STRIP(v_lowerBoundStr, BOTH, '''');



  --
  -- reformat the datapartitions.highvalue column into a format the timestamp_format
  --  function can successfully digest.
  --
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  SET v_workStr =
       substr(v_upperBoundStr,1,4) || '-' ||
       substr(v_upperBoundStr,6,2) || '-' ||
       substr(v_upperBoundStr,9,2) || ' ' ||
       --substr(v_upperBoundStr,12,2) || ':' ||
       --substr(v_upperBoundStr,15,2) || ':' ||
       --substr(v_upperBoundStr,18,2) ;
       '00:00:00' ;



  SET v_timestamp = timestamp( v_workStr );

  --
  -- apply the correct granularity unit
  --
  
  IF ( v_granularity_unit = 'HOUR' ) THEN
    SET v_timestamp = v_timestamp + v_granularity HOUR ;
  ELSEIF ( v_granularity_unit = 'MONTH' ) THEN
    SET v_timestamp =  v_timestamp - day(v_timestamp) DAYS + 1 DAY + 2 MONTH - 1 DAY;
  ELSE
    SET v_timestamp = v_timestamp + v_granularity DAY  ;
  END IF ;




  --
  -- timestamp_format and to_char don't recognize more the one mask 
  --   so kludge around the shortcomming 
  -- 
  SET v_newupperBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;
  SET v_newupperBoundStr = v_newupperBoundStr || '.999999' ;
  SET v_newupperBoundStr = DATE ( v_newupperBoundStr ) ;




  --
  -- reformat the datapartitions.lowvalue column into a format the timestamp_format
  --  function can successfully digest.
  --
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  SET v_workStr =
       substr(v_lowerBoundStr,1,4) || '-' ||
       substr(v_lowerBoundStr,6,2) || '-' ||
       substr(v_lowerBoundStr,9,2) || ' ' ||
       --substr(v_lowerBoundStr,12,2) || ':' ||
       --substr(v_lowerBoundStr,15,2) || ':' ||
       --substr(v_lowerBoundStr,18,2) ;
       '23:59:59';


  SET v_timestamp = timestamp( v_workStr );
  SET v_timestamp = v_timestamp + 1 SECOND ;
  SET v_newlowerBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;
  SET v_newlowerBoundStr = DATE ( v_newlowerBoundStr ) ;



  --
  -- extract part#, part##, part###, or partyyyymmdd from the table name to use as the partition name
  --
  --SET v_partition = substr(v_table_part_name, 15, 5 );
  SET v_partition = rtrim( substr(v_table_part_name, posstr(v_table_part_name, 'PART'), 
                           (length(v_table_part_name) - posstr(v_table_part_name, 'PART')+1)) );

  --
  -- attach it
  -- 
  SET v_dynStmt = 'ALTER TABLE ' || i_schema || '.' ||  i_tabname ||' ATTACH PARTITION ' 
                  || ''  || v_partition || ''  
                  || ' STARTING ' || '''' || v_newlowerboundStr || ''''
                  || ' ENDING '   || '''' || v_newupperboundStr || ''''
                  || ' FROM TABLE ' || i_schema || '.' || v_table_part_name
                ;


  CALL exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );


  IF ( o_sqlcode != 0 ) THEN
    RETURN ;
  END IF;

  CALL set_integrity_on(i_tabname, i_schema, i_preview_only_ind, i_locktimeout_cnt, o_sqlcode );
 

END P1
@
CREATE PROCEDURE DETACH_PARTITION ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30), 
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    IN i_detach_wait_sec   INTEGER,
                                    OUT o_detached_table  VARCHAR(50),
                                    OUT o_sqlcode INT  )
SPECIFIC DETACH_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  DETACH_PARTITION
--
--
-- Description:
--
--     This version of the proc WILL ONLY detach partitions part1 through part##.
--
--     Given the name of a partioned table detach a partition into a 
--     free standing table.  The table name will be of the form: FCM_XXXX_DTCH_PART## .
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/28  created       
--      2007/11/29  added the schema name to qualify the detached table
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log procedure 
--      2012/08/23   leushe   Added support for the DETACH_WAIT_SEC (i_detach_wait_sec)
--                            to check for detach partition procedure completion due to 9.7 FP1 
--                            Asynchronous Detach Opertaion--            
--           
--          
--         
--        
--	 

--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;
  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the name of the partition with the minimum lowvalue that isn't the CF partition
  -- 
  SET v_partition = 
      ( select cast(datapartitionname as varchar(50))  
        from syscat.datapartitions a
        where ( tabname = i_tabname and datapartitionname not like '%CF%' 
                          and  datapartitionname not like '%THE_END%' 
                          and tabschema = i_schema 
                          and status != 'I' )
        and tabschema = i_schema 
        and lowvalue = ( select min(lowvalue) 
                         from syscat.datapartitions b
                         where a.tabschema = b.tabschema 
                         and a.tabname = b.tabname 
                         and b.datapartitionname not like '%CF%' 
                         and b.datapartitionname not like '%THE_END%' 
                         and tabschema = i_schema 
                         and status != 'I' )
      );



  --
  -- create the detached table name
  --
  SET v_table_part_name = i_tabname || '_DTCH_' || v_partition ;


  --
  -- save the name of the detached table for the client
  --
  SET o_detached_table = v_table_part_name ; 
 

  --
  -- create the dynamic sql to detach the partition
  -- 
  SET v_dynStmt = 'ALTER TABLE ' || i_schema || '.' ||  i_tabname ||' DETACH PARTITION ' 
                  || ''  || v_partition || ''  
                  || ' INTO ' || i_schema || '.' || v_table_part_name  || ''
                ;


  call exec_sql_and_log (i_tabname, i_schema, v_partition, i_preview_only_ind,i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN;
  END IF;
    set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'
                  );
  END WHILE;
END P1
@

CREATE PROCEDURE truncate         ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    OUT o_sqlcode INT  )
SPECIFIC truncate         
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  truncate
--
--
-- Description:
--     Given the name of a table truncate it by invoking the loader.
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT 
--                            to pass to exec_sql_and_log procedure 
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--      2012/08/23   leushe   reverted previous change. Moved sleep to DETACH_PARTITION
--            
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_dynStmtParm     VARCHAR(1024);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer;
  DECLARE v_sqlstate char(5) ;
  
  DECLARE v_stmt STATEMENT;


  --
  -- build the dynamic truncate statement via import /dev/null
  --
  SET v_dynStmtParm = 'IMPORT FROM /dev/null of del replace into '||i_schema||'.'||i_tabname;

  --
  -- create the dynamic sql statement
  -- 
  SET v_dynStmt = 'call sysproc.admin_cmd (?)';
  

  CALL exec_sql_and_log 
        (i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt,v_dynStmt, v_dynStmtParm, o_sqlcode) ;


END P1
@


CREATE PROCEDURE roll_partition ( IN i_tabname          VARCHAR(50),
                                     IN i_schema           VARCHAR(30)
                                       )
SPECIFIC ROLL_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ROLL_PARTITION
--
--
-- Description:
--     Top level entry point for the RH partitioning maintenance.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/26   jicimi   created       
--      2007/11/05   jicimi   added insert into frx_partition_definition
--                            to control how often the code is allowed to  
--                            run - sanity_check queries this column
--      2009/08/19   leushe   renamed the procedure to roll_partition
--							  changed the call to attach_rh_partition to the procedures
--                            new name ATTACH_PARTITION
--      2009/08/24   leushe   Changed the proc to not return explicit parameters, but return 
--                            the sqlcode via a result set. this changed the signature of the proc
--                            This change was made so SAS could call the proc, as it doesn't handle explicit
--                            Output parameters
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT Column on the partition_definition table 
--      2012/04/26   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the truncate proc
--      2012/08/23   leushe   Reverted previous change, added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the detach_partition proc
--    
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_rowCount        INTEGER   DEFAULT 0;
  DECLARE v_detached_table  VARCHAR(50);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0;
  DECLARE v_sqlstate varchar(50) DEFAULT '00000' ;
  DECLARE v_sqlmsg varchar(30000)  ;
  DECLARE l_preview_only_ind CHAR(1) DEFAULT '0';
  DECLARE l_chk_sanity_ind CHAR(1) DEFAULT '0';
  DECLARE l_locktimeout_cnt INTEGER DEFAULT 3;
  DECLARE l_detach_wait_sec integer DEFAULT 30;
  
      /* Declare cursor to return result set to calling program */ 
        
        DECLARE c_result CURSOR WITH RETURN FOR
            SELECT   v_sqlcode as OUT_SQLCODE
                    ,v_sqlstate as OUT_SQLSTATE
            FROM    SYSIBM.SYSDUMMY1;
  
      /* Initialize variables */
        VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;

 
  -- 1.  upcase all arguments 
  SET i_tabname = upper(i_tabname);
  SET i_schema = upper(i_schema);
  SET v_sqlstate = i_tabname ;
 
  --
  -- verify i_tabname is indeed a partitioned table
  --
  SET v_rowCount = ( select count(*) from frx_partition_definition where table_name = i_tabname );

  IF ( v_rowCount != 1 ) THEN
    SET v_sqlcode = -1;
    SET v_sqlstate = '-1';
    SET v_sqlmsg = '';
    CALL audit_partition_action(i_tabname, i_schema, NULL, i_tabname||' is not partition eligible.',
                                '0', v_sqlcode, v_sqlstate,v_sqlmsg );
          
    SET v_sqlstate = i_tabname;                                
    OPEN c_result;

    RETURN -1;
  END IF;
 
  Select LOCKTIMEOUT_RETRY_COUNT, PREVIEW_ONLY_IND,SKIP_SANITY_CHECK_IND,DETACH_WAIT_SEC 
  		INTO l_locktimeout_cnt,l_preview_only_ind,l_chk_sanity_ind,l_detach_wait_sec
  FROM FRX_PARTITION_DEFINITION
  where table_name = i_tabname;
  
 
  IF ( l_chk_sanity_ind = '0' ) THEN
    -- 2. sanity check its ok to go on
    call sanity_check_partition(i_tabname, i_schema, l_preview_only_ind, v_sqlcode) ;

    IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
   END IF; 
  END IF;
 

  -- 3. attach the detached partiton as the new one
  call ATTACH_PARTITION(i_tabname, i_schema, l_preview_only_ind,l_locktimeout_cnt, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
  END IF;



  -- 4. detach oldest
  call detach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_detach_wait_sec,v_detached_table,v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
  END IF;

   
  -- 5. empty oldest
  call truncate(v_detached_table, i_schema, l_preview_only_ind,l_locktimeout_cnt,v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
       OPEN c_result;

      RETURN -1;
  END IF;


  -- 6. insert current timestamp into definition table
  --    this value is used to ensure the code does not
  --    execute more frequently than it is supposed to
     IF ( l_preview_only_ind = '0' ) THEN
        update frx_partition_definition 
         set last_run_timestamp = ( current timestamp )
         where table_name = i_tabname ;
     END IF;
  
  VALUES ( SQLCODE) INTO  v_sqlcode;
  COMMIT;
  OPEN c_result;  
END P1
@


