call sysproc.set_routine_opts('ISOLATION UR') @



CREATE PROCEDURE ATTACH_PARTITION ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30), 
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    IN i_partition_datatype CHAR(9),
                                    OUT o_sqlcode INT  )
SPECIFIC ATTACH_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ATTACH_PARTITION
--
--
-- Description:
--     This should be the normal path through the attach code for all non CF
--     partitions.
--
--     Given the name of a partioned table & a free standing table
--     attach the free standing table as a new partition.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/29   jicimi   created       
--      2007/12/07   jicimi   corrected substr command at line 232
--      2009/07/15   leushe   change the upper bound to be the last day of the month
--                            when partitioing by MONTH
--      2009/08/19   leushe   renamed to attach_partition 
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log procedure    
--      2012/07/13   leushe   Fixed the upper bound calculation for Month Partitioning 
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name    VARCHAR(50);
  DECLARE v_partition          VARCHAR(50);
  DECLARE v_dynStmt            VARCHAR(1024);
  DECLARE v_dateStr            VARCHAR(30);
  DECLARE v_lowerBoundStr      VARCHAR(30);
  DECLARE v_upperBoundStr      VARCHAR(30);
  DECLARE v_newlowerBoundStr   VARCHAR(30);
  DECLARE v_newupperBoundStr   VARCHAR(30);
  DECLARE v_workStr            VARCHAR(30);
  DECLARE v_timestamp          TIMESTAMP;
  DECLARE uniqueStr            VARCHAR(10);
  DECLARE v_granularity        INTEGER;
  DECLARE v_granularity_unit   CHAR(10);


  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the detached table/partition name
  -- 
  SET v_table_part_name = 
      ( select tabname from syscat.tables a
        where tabname like i_tabname || '_DTCH_PART%' 
        and tabschema = i_schema
        order by create_time 
        fetch first row only
      );


  --
  -- to assign the new lowerbound 
  -- get the current upperbound of the last NON CF partition
  -- add 1 second to that value and that becomes the lowerbound of the new partition
  -- this essentially anchors the lowerbound so no gaps are introduced 
  -- if the partition definition table gets updated on the fly
  --
  -- to assign the new upperbound
  -- get the current upperbound of the last NON CF partition
  -- get the granularity value for the table from the partition definition table
  -- add the two togther and that becomes the new upperbound 
  -- 
  --

  --
  -- get the current upperbound of the last NON CF partition
  -- 
  SET v_lowerBoundStr =
      ( select cast(max(highvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname
        and datapartitionname like 'PART%'
        and datapartitionname not like '%THE_END%'
        and tabschema = i_schema
      );


  --
  -- get the current upperbound of the last NON CF partition
  --
  SET v_upperBoundStr =
      ( select cast(max(highvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname
        and datapartitionname like 'PART%'
        and datapartitionname not like '%THE_END%'
        and tabschema = i_schema
      );



  --
  -- get the increment for the upper partition bound
  --
  SET v_granularity = 
      ( select granularity 
        from fmx_partition_definition 
        where table_name = i_tabname 
      );


  --
  -- get the granularity unit ( HOUR, DAY, MONTH )  for the upper partition bound
  --
  SET v_granularity_unit = 
      ( select granularity_unit 
        from fmx_partition_definition 
        where table_name = i_tabname 
      );


  --
  -- get the new upperBoundStr & lowerBoundStr values
  --    take the str representation from the catalog
  --    convert it to a date/timestamp
  --    the # of days to add should be driven by the metadata table
  --        add the granularity using date arihematic 
  --    then convert the timestamp back to a string for the attach call
  --

  SET v_upperBoundStr = STRIP(v_upperBoundStr, BOTH, '''');
  SET v_lowerBoundStr = STRIP(v_lowerBoundStr, BOTH, '''');



  --
  -- reformat the datapartitions.highvalue column into a format the timestamp_format
  --  function can successfully digest.
  --
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  IF (i_partition_datatype = 'TIMESTAMP') THEN
  SET v_workStr =
       substr(v_upperBoundStr,1,4) || '-' ||
       substr(v_upperBoundStr,6,2) || '-' ||
       substr(v_upperBoundStr,9,2) || ' ' ||
       substr(v_upperBoundStr,12,2) || ':' ||
       substr(v_upperBoundStr,15,2) || ':' ||
       substr(v_upperBoundStr,18,2) ;
  ELSEIF (i_partition_datatype = 'DATE') THEN
  SET v_workStr =
       substr(v_upperBoundStr,1,4) || '-' ||
       substr(v_upperBoundStr,6,2) || '-' ||
       substr(v_upperBoundStr,9,2) || ' ' ||
       --substr(v_upperBoundStr,12,2) || ':' ||
       --substr(v_upperBoundStr,15,2) || ':' ||
       --substr(v_upperBoundStr,18,2) ;
       '00:00:00' ;
  END IF;

  SET v_timestamp = timestamp( v_workStr );

  --
  -- apply the correct granularity unit
  --
  -- SET v_upperbound = i_lowerbound + v_granularity DAY  ;
  IF ( v_granularity_unit = 'HOUR' ) THEN
    SET v_timestamp = v_timestamp + v_granularity HOUR ;
  ELSEIF ( v_granularity_unit = 'MONTH' ) THEN
    SET v_timestamp =  v_timestamp - day(v_timestamp) DAYS + 1 DAY + 2 MONTH - 1 DAY;
  ELSE
    SET v_timestamp = v_timestamp + v_granularity DAY  ;
  END IF ;




  --
  -- timestamp_format and to_char don't recognize more the one mask 
  --   so kludge around the shortcomming 
  -- 
  SET v_newupperBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;
  SET v_newupperBoundStr = v_newupperBoundStr || '.999999' ;
  IF (i_partition_datatype = 'DATE') THEN
  	SET v_newupperBoundStr = DATE ( v_newupperBoundStr ) ;
  END IF;


  --
  -- reformat the datapartitions.lowvalue column into a format the timestamp_format
  --  function can successfully digest.
  --
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  IF (i_partition_datatype = 'TIMESTAMP') THEN
     SET v_workStr =
       substr(v_lowerBoundStr,1,4) || '-' ||
       substr(v_lowerBoundStr,6,2) || '-' ||
       substr(v_lowerBoundStr,9,2) || ' ' ||
       substr(v_lowerBoundStr,12,2) || ':' ||
       substr(v_lowerBoundStr,15,2) || ':' ||
       substr(v_lowerBoundStr,18,2) ;
  ELSEIF (i_partition_datatype = 'DATE') THEN
  SET v_workStr =
       substr(v_lowerBoundStr,1,4) || '-' ||
       substr(v_lowerBoundStr,6,2) || '-' ||
       substr(v_lowerBoundStr,9,2) || ' ' ||
       --substr(v_lowerBoundStr,12,2) || ':' ||
       --substr(v_lowerBoundStr,15,2) || ':' ||
       --substr(v_lowerBoundStr,18,2) ;
       '23:59:59';
  END IF;

  SET v_timestamp = timestamp( v_workStr );
  SET v_timestamp = v_timestamp + 1 SECOND ;
  SET v_newlowerBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;
  IF (i_partition_datatype = 'DATE') THEN
    SET v_newlowerBoundStr = DATE ( v_newlowerBoundStr ) ;
  END IF;
  

  --
  -- extract part#, part##, part###, or partyyyymmdd from the table name to use as the partition name
  --
  --SET v_partition = substr(v_table_part_name, 15, 5 );
  SET v_partition = rtrim( substr(v_table_part_name, posstr(v_table_part_name, 'PART'),
                           ( length(v_table_part_name) - posstr(v_table_part_name, 'PART')+1) ));


  --
  -- attach it
  -- 
  SET v_dynStmt = 'ALTER TABLE ' || i_schema || '.' ||  i_tabname ||' ATTACH PARTITION ' 
                  || ''  || v_partition || ''  
                  || ' STARTING ' || '''' || v_newlowerboundStr || ''''
                  || ' ENDING '   || '''' || v_newupperboundStr || ''''
                  || ' FROM TABLE ' || i_schema || '.' || v_table_part_name
                ;

  CALL exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );


  IF ( o_sqlcode != 0 ) THEN
    RETURN ;
  END IF;

  CALL set_integrity_on(i_tabname, i_schema, i_preview_only_ind,i_locktimeout_cnt, o_sqlcode );
 

END P1
@











CREATE PROCEDURE DETACH_PARTITION ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30), 
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,
                                    IN i_detach_wait_sec   INTEGER,
                                    OUT o_detached_table  VARCHAR(50),
                                    OUT o_sqlcode INT  )
SPECIFIC DETACH_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  DETACH_PARTITION
--
--
-- Description:
--
--     This version of the proc WILL ONLY detach partitions part1 through part##.
--
--     Given the name of a partioned table detach a partition into a 
--     free standing table.  The table name will be of the form: FCM_XXXX_DTCH_PART## .
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/28  created       
--      2007/11/29  added the schema name to qualify the detached table
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log procedure
--      2012/08/17   leushe   Added support for the DETACH_WAIT_SEC (i_detach_wait_sec)
--                            Added Check to see if the table being detached does not have a partition with status= 'L' 
--                            in SYSCAT.DATAPARTITIONS
--            
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;
  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the name of the partition with the minimum lowvalue that isn't the CF partition
  -- 
  SET v_partition = 
      ( select cast(datapartitionname as varchar(50))  
        from syscat.datapartitions a
        where ( tabname = i_tabname and datapartitionname not like '%CF%' 
                          and  datapartitionname not like '%THE_END%' 
                          and tabschema = i_schema 
                          and status != 'I' )
        and tabschema = i_schema 
        and lowvalue = ( select min(lowvalue) 
                         from syscat.datapartitions b
                         where a.tabschema = b.tabschema 
                         and a.tabname = b.tabname 
                         and b.datapartitionname not like '%CF%' 
                         and b.datapartitionname not like '%THE_END%' 
                         and tabschema = i_schema 
                         and status != 'I' )
      );



  --
  -- create the detached table name
  --
  SET v_table_part_name = i_tabname || '_DTCH_' || v_partition ;


  --
  -- save the name of the detached table for the client
  --
  SET o_detached_table = v_table_part_name ; 
 

  --
  -- create the dynamic sql to detach the partition
  -- 
  SET v_dynStmt = 'ALTER TABLE ' || i_schema || '.' ||  i_tabname ||' DETACH PARTITION ' 
                  || ''  || v_partition || ''  
                  || ' INTO ' || i_schema || '.' || v_table_part_name  || ''
                ;


  call exec_sql_and_log (i_tabname, i_schema, v_partition, i_preview_only_ind,i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN;
  END IF;

    set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'
                  );
  END WHILE;
  
END P1
@

 



CREATE PROCEDURE DETACH_PARTITION ( IN i_tabname   VARCHAR(50),
                                    IN i_schema    VARCHAR(30), 
                                    IN i_partition VARCHAR(50),   -- used as a flag 
                                                                  -- if 'CF' then 
                                                                  -- detach the CF# partition
                                                                  --  else 
                                                                  -- deatch the eariliest non 
                                                                  -- cf partition 
                                    IN i_preview_only_ind   CHAR(1),
                                    IN i_locktimeout_cnt   INTEGER,
                                    IN i_detach_wait_sec   INTEGER,
                                    OUT o_detached_table  VARCHAR(50),
                                    OUT o_sqlcode  INT  )
SPECIFIC DETACH_PARTITION_NAME
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  DETACH_PARTITION
--
--
-- Description:
--     This is the normal path through the detach code.
--
--     Given the name of a partioned table detach a partition into a 
--     free standing table. 
-- 
--     The table name will be of the form: FCM_XXXX_DTCH_PART## if the i_partition
--     argument doens't contain 'CF' otherwise the tablename will be of the form
--     FCM_XXX_DTCH_PART_CF# .
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/28  created       
--      2007/08/30  added logic to use partition name as a flag      
--      2007/11/29  added the schema name to qualify the detached table
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log procedure
--      2012/08/17   leushe   Added support for the DETACH_WAIT_SEC (i_detach_wait_sec)
--                            Added Check to see if the table being detached does not have a partition with status= 'L' 
--                            in SYSCAT.DATAPARTITIONS
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_invalid_part_cnt integer DEFAULT 0 ;
  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;




  IF ( POSSTR(i_partition, 'CF') > 0 ) THEN

    --
    -- get the name of the CF partition 
    --
    SET v_partition =
        ( select cast(datapartitionname as varchar(50))
          from syscat.datapartitions a
          where tabname = i_tabname 
            and datapartitionname like '%CF%'
            and tabschema = i_schema 
            and status != 'I'
        );

  ELSE

    --
    -- get the name of the partition with the minimum lowvalue that isn't the CF partition
    --
    SET v_partition =
        ( select cast(datapartitionname as varchar(50))
          from syscat.datapartitions a
          where ( tabname = i_tabname and datapartitionname not like '%CF%'
                            and  datapartitionname not like '%THE_END%' 
                            and tabschema = i_schema 
                            and status != 'I' )
          and tabschema = i_schema
          and lowvalue = ( select min(lowvalue)
                           from syscat.datapartitions b
                           where a.tabschema = b.tabschema
                           and a.tabname = b.tabname
                           and b.datapartitionname not like '%CF%'
                           and b.datapartitionname not like '%THE_END%' 
                           and b.tabschema = i_schema 
                           and status != 'I' )
        );

  END IF ;



  --
  -- create the detached table name
  --
  SET v_table_part_name = i_tabname || '_DTCH_' || v_partition ;


  --
  -- save the name of the detached table for the client
  --
  SET o_detached_table = v_table_part_name ; 

  --
  -- create the dynamic sql to detach the partition
  -- 
  SET v_dynStmt = 'ALTER TABLE ' || i_schema || '.' ||  i_tabname ||' DETACH PARTITION ' 
                  || ''  || v_partition || ''  
                  || ' INTO ' || i_schema || '.' || v_table_part_name  || ''
                ;

  call exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt,v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN ;
  END IF;
  
  set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'  
                  );

  WHILE (v_invalid_part_cnt > 0 ) DO

        call sleep(i_detach_wait_sec);
  		set v_invalid_part_cnt = ( select count(*)  
          from syscat.datapartitions 
          where  tabname = i_tabname 
          		and tabschema = i_schema 
            	and status = 'L'
                  );
  END WHILE;

END P1
@


CREATE PROCEDURE UPDATE_CF_PARTITION ( IN i_tabname   VARCHAR(50),
                                       IN i_schema    VARCHAR(30), 
                                       IN i_cf_partition VARCHAR(50),        -- the cf partition
                                       IN i_preview_only_ind   CHAR(1),
                                       IN i_locktimeout_cnt   INTEGER,
                                       IN i_partition_datatype CHAR(9),
                                       IN i_detach_wait_sec INTEGER,
                                       OUT o_sqlcode  INT  )
SPECIFIC UPDATE_CF_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  UPDATE_CF_PARTITION
--
--
-- Description:  
--     This should be the normal code path.   
--
--     Given the name of a partioned table detach the cf partition and copy
--     the eligible rows from the temp_part# partition into the cf partition.
--
--     This version of the proc determines the detached partition rather than
--     accepting it as a parameter.
--      
--     Obviously a partition had better be detached before getting here.
--
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--           
--
-- Usage:
--
--
-- History:    
--      2007/08/30   jicimi   created       
--      2007/11/05   jicimi   line 285 commented out 
--                              SET v_timestamp = v_timestamp - 1 MICROSECOND;
--      2007/11/22   jicimi   global temp table changes
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                            to pass to exec_sql_and_log and set_integrity procedure 
--      2012/04/25   leushe   changed the query to retrieve the CF partition new upper bound
--                            to check partitions where the status value is empty
--                            length(trim(status))=0
--                            Also added a loop to check to make sure the status is empty
--                            for all partitions, if not wait 30 seconds and check the status column
--                            for all partitions, if not wait 30 seconds again
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--      2012/08/17   leushe   reverted change from 2012/04/25. The change on 2012/04/26 was kept and the 
--                            i_detach_wait_sec parameter is now sent to the Detach Partition proc, when the code 
--                            from 2012/04/25 was moved to.
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name    VARCHAR(50);
  DECLARE v_partition          VARCHAR(50);
  DECLARE v_dynStmt            VARCHAR(1024);
  DECLARE v_lowerBoundStr      VARCHAR(30);
  DECLARE v_upperBoundStr      VARCHAR(30);
  DECLARE v_newUpperBoundStr   VARCHAR(30);
  DECLARE v_dayofweek          CHAR(1) ; 
  DECLARE v_newpartname        VARCHAR(50);     -- this is the name of the detached cf partition
  DECLARE v_cf_partition_name  VARCHAR(50);     -- this is the name of the detached cf table
  DECLARE v_workStr            VARCHAR(50);
  DECLARE v_workStr2           VARCHAR(50);
  DECLARE v_timestamp          TIMESTAMP;
  DECLARE v_date               DATE;
  DECLARE v_detached_partition VARCHAR(50);
  DECLARE o_detached_partition VARCHAR(50);
  DECLARE v_temp_table         VARCHAR(50);     -- the name of the gloabl temp table



  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;

  DECLARE v_stmt STATEMENT;



  -- 
  -- get the name of the CF partition 
  --
  -- ******
  -- ******
  -- ******
  -- Should the i_cf_partition parameter be removed ?
  -- ******
  -- ******
  -- ******
  -- 
  SET v_partition = 
      ( select cast(datapartitionname as varchar(50))  
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname like '%CF%' 
        and tabschema = i_schema 
        and status != 'I'
      );


  -- 
  -- get the current min(lowerbound) of the 1st non CF partition 
  -- to use as the upperbound of the CF partition
  -- 
  -- 2007/09/16 - in preview mode, this creates a bug
  --              the oldest partition is never detached so the min(lowvalue) 
  --              value returned is actually from the partition that under ordinary
  --              cirsumstances would have been detached 
  --
  --              the following if compensates for that shortcomming
  --      
  SET v_upperBoundStr = 
      ( select cast( min(lowvalue) as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname not like '%CF%'  
        and tabschema = i_schema 
        and length(trim(status))=0
      );

       
      
  IF ( i_preview_only_ind = '1' ) THEN
    SET v_upperBoundStr = 
        ( select cast( min(lowvalue) as varchar(30))
          from syscat.datapartitions a
          where tabname = i_tabname 
          and datapartitionname not like '%CF%'  
          and datapartitionname not like '%THE_END%'  
          and tabschema = i_schema 
          and status != 'I'
          and lowvalue > 
               ( select  min(lowvalue) 
                 from syscat.datapartitions b
                 where tabname = i_tabname 
                 and datapartitionname not like '%CF%'  
                 and datapartitionname not like '%THE_END%'  
                 and tabschema = i_schema 
                 and length(trim(status))=0 )
        );
  END IF;


  -- 
  -- get the current lowerbound  of the CF partition 
  -- 
  SET v_lowerBoundStr = 
      ( select cast(lowvalue as varchar(30))
        from syscat.datapartitions a
        where tabname = i_tabname 
        and datapartitionname like '%CF%' 
        and tabschema = i_schema 
        and status != 'I'
      );



  -- 
  -- get the name of the detached partition fcm_xxxx_temp_part#,  where # in 1..##
  --
  -- 2007/09/10 - in preview mode, this creates a bug 
  --              the oldest partition is never detached so the insert statement
  --              uses the name of the currently detached partition rather than the
  --              newly detached one 
  --
  --              resolved with the following if statement 
  --
  SET v_detached_partition =
      ( select cast(tabname as varchar(50))
        from syscat.tables a
        where ( tabname like i_tabname||'_DTCH_PART%' 
                and tabname not like i_tabname||'_DTCH_%CF%' ) 
        and tabschema = i_schema 
        and status != 'I'
      );
     

  IF ( i_preview_only_ind = '1' ) THEN
    SET v_detached_partition = 
        ( select i_tabname || '_DTCH_' || cast(datapartitionname as varchar(50))  
          from syscat.datapartitions a
          where ( tabname = i_tabname and datapartitionname not like '%CF%' 
                            and  datapartitionname not like '%THE_END%' 
                            and tabschema = i_schema 
                            and status != 'I' )
          and tabschema = i_schema 
          and lowvalue = ( select min(lowvalue) 
                           from syscat.datapartitions b
                           where a.tabschema = b.tabschema 
                           and a.tabname = b.tabname 
                           and b.datapartitionname not like '%CF%' 
                           and b.datapartitionname not like '%THE_END%' 
                           and tabschema = i_schema 
                           and status != 'I' )
        );
  END IF;   
  



  --
  --  get the dayofweek # to append to the cf partition name
  --  ( to allow the cf partition to be reattached )
  --
  SET v_dayOfWeek = ( select cast( dayofweek(current date) as char(1)) from sysibm.sysdummy1 ) ;


  --
  -- rename the cf partition to avoid the SQL0612N dup name, 42711 error
  --   always of the form PART_CF#
  --
  SET v_newpartname = substr(v_partition,1,7);
  SET v_newpartname = v_newpartname || v_dayOfWeek;
  
  --
  --
  --
  --  *******
  --  *******
  --  *******
  --  just for testing to avoid the SQL0612N dup name, SQLSTATE=42711 error
  --     append the current microsecond value onto the partition name 
  --  *******
  --  *******
  --  *******
  --
  SET v_newpartname = v_newpartname || '_' || 
      cast( microsecond(current timestamp) as char(6));
  --
  --  *******
  --  *******
  --  *******
  --  end test code
  --  *******
  --  *******
  --  *******
  --





  
  --
  -- the attach will fail if minvalue is used, so reset it if need be
  --
  IF ( v_lowerBoundStr = 'MINVALUE' ) THEN
     SET v_lowerBoundStr = '2006-01-01 00:00:00';
  END IF;


  
  --
  -- get the new upperBoundStr value
  --    take the str representation from the catalog    
  --    convert it to a date/timestamp
  --    then convert it back to a string for the attach call
  --

  SET v_upperBoundStr = STRIP(v_upperBoundStr, BOTH, '''');
  SET v_lowerBoundStr = STRIP(v_lowerBoundStr, BOTH, '''');

 

  --
  -- reformat the datapartitions.highvalue column into a format the timestamp_format
  --  function can sucessfully digest.
  -- 
  -- If the timestamp_format function only accepts one mask, WHY is the mask REQUIRED?!
  --
  SET v_workStr = 
       substr(v_upperBoundStr,1,4) || '-' || 
       substr(v_upperBoundStr,6,2) || '-' ||
       substr(v_upperBoundStr,9,2) || ' ' ||
       substr(v_upperBoundStr,12,2) || ':' || 
       substr(v_upperBoundStr,15,2) || ':' ||
       substr(v_upperBoundStr,18,2) ;


 
  SET v_timestamp = timestamp( v_workStr );
  -- 2007/11/05  commented out
  -- SET v_timestamp = v_timestamp - 1 MICROSECOND;
  SET v_newupperBoundStr = TO_CHAR(v_timestamp, 'YYYY-MM-DD HH24:MI:SS') ;

  


  -- 
  --  detach cf partition
  --
  CALL detach_partition(i_tabname, i_schema, v_partition, 
                        i_preview_only_ind, i_locktimeout_cnt, i_detach_wait_sec, o_detached_partition, o_sqlcode );


  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     

  --
  -- build the name of the newly detached cf table
  --
  SET v_cf_partition_name = o_detached_partition;

  
  --
  -- get the name of the global temp table
  --
  SET v_temp_table = i_tabname ||'_GTT' ;
  
 
  --
  -- insert eligible rows from DTCH_PART# => ( v_detached_partition )  
  --      into DTCH_CF# => ( i_cf_partition )
  --
  CALL copy_cf_rows (i_tabname, i_schema, v_temp_table,
                     v_cf_partition_name, i_preview_only_ind,i_locktimeout_cnt, /*10000,*/ o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     

  --
  -- attach cf partition with new upper BoundStr
  --
  CALL attach_partition(i_tabname, i_schema, v_lowerBoundStr, v_newupperBoundStr, 
                        v_partition, v_newpartname, i_preview_only_ind,i_locktimeout_cnt,i_partition_datatype, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
    RETURN;
  END IF;
     


  --
  -- set the ri back on
  --
  CALL set_integrity_on ( i_tabname, i_schema, i_preview_only_ind,i_locktimeout_cnt, o_sqlcode );


END P1
@


CREATE PROCEDURE truncate         ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30),
                                    IN i_preview_only_ind CHAR(1),
                                    IN i_locktimeout_cnt INTEGER,

                                    OUT o_sqlcode INT  )
SPECIFIC truncate         
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  truncate
--
--
-- Description:
--     Given the name of a table truncate it by invoking the loader.
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2007/09/06   jicimi   created       
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT 
--                            to pass to exec_sql_and_log procedure 
--      2012/04/26   leushe   added new parameter i_detach_wait_sec to send to the sleep() call
--                            The value comes from the FMX_PARTITION_DEFINITION table.
--      2012/08/17   leushe   reverted change from 2012/04/26. The check for Detach partition completion
--                            is now located in the DETACH_PARTITION Procedure
--            
--           
--          
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_dynStmtParm     VARCHAR(1024);
  

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer;
  DECLARE v_sqlstate char(5) ;
  
  DECLARE v_stmt STATEMENT;


  --
  -- build the dynamic truncate statement via import /dev/null
  --
  SET v_dynStmtParm = 'IMPORT FROM /dev/null of del replace into '||i_schema||'.'||i_tabname;

  --
  -- create the dynamic sql statement
  -- 
  SET v_dynStmt = 'call sysproc.admin_cmd (?)';
  
  

  CALL exec_sql_and_log 
        (i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt,v_dynStmt, v_dynStmtParm, o_sqlcode) ;


END P1
@

CREATE PROCEDURE CREATE_PARTITION_TEMP_TABLE
                                   ( IN i_tabname   VARCHAR(50),
                                     IN i_schema    VARCHAR(30), 
                                     IN i_preview_only_ind   CHAR(1),
                                     IN i_locktimeout_cnt INTEGER,
                                     OUT o_temp_table  VARCHAR(50),     -- the name of the _gtt
                                     OUT o_sqlcode  INT  )
SPECIFIC CREATE_PART_GTT
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  CREATE TEMP TABLE
--
--
-- Description:
--     Given the name of a partioned table find the oldest partition, 
--     create a global temp table, and copy all eligible rows from the oldest partition
--     into the global temp table.
-- 
--
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
--    The insert statement used to populate the global temporary table changes 
--    between R1 and R2 bear that in mind.  Both versions are resident, but 
--    one is commented out.
--
--    Also this proc expects a user 32k temp tablespace to exist and be accessible.
--    
--           
-- Usage:
--   call create_partition_temp_table ('FCM_AUTH', 'FCMCORE', '0', ?, ?) ;
--
--
-- History:    
--      2007/11/22  created       
--      2009/06/18  Updated for V3. Looks at xqo_cust_num = grx_cust_num, instead of acct_num       
--      2009/09/03  leushe  Added support for the LOCKTIMEOUT_RETRY_COUNT (i_locktimeout_cnt)
--                          to pass to exec_sql_and_log procedure
--      2010/10/22  leushe  Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively--           
--      2010/12/22  leushe  fixed ambigous reference to xqo_Cust_num
--      2011/01/27  leushe  changed the preservation of CF txn logic to determine which "alert type" fields
--                          are on the txn table, and compare those columns with the alert_value of the alert
--                          by alert_type    
--      2012/09/11  leushe  changed the preservation of CF txn logic to determine which "alert type" fields
--                          to use an OR clause
--         
--        
--	 
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_table_part_name VARCHAR(50);
  DECLARE v_partition       VARCHAR(50);
  DECLARE v_dynStmt         VARCHAR(1024);
  DECLARE v_lowerbound      VARCHAR(36);
  DECLARE v_upperbound      VARCHAR(36);
  DECLARE v_user_tablespace VARCHAR(64);
  DECLARE v_temp_table      VARCHAR(50);
  DECLARE l_counter INTEGER DEFAULT 0; 
  DECLARE l_prepStmtTemplate varchar(1000);
  DECLARE at_end SMALLINT DEFAULT 0;    
  DECLARE v_fieldname VARCHAR(50);
  DECLARE v_alert_type CHAR(1); 
  
  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE not_found CONDITION FOR SQLSTATE '02000';
  
  DECLARE v_stmt STATEMENT;

--' AND ( alrt.alert_Value = txn.'||rtrim(field_name)||'  and alrt.alert_type = '''||alert_Type||''')' as xyz
    DECLARE c_prep CURSOR FOR
	  select distinct field_name,alert_Type
            from fcm_alert_type at, fcm_field_definition fd, fcm_transaction_Definition td,
	    FCM_transaction_component tc, fcm_component_segment cs,fcm_segment_Definition sd,fcm_component_value cv
	  where fd.field_id = at.KEY_FIELD_ID
          and sd.segment_id = fd.SEGMENT_ID
          and cs.SEGMENT_ID = sd.SEGMENT_ID
          and tc.TRANSACTION_ID = td.TRANSACTION_ID
          and tc.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and cs.COMPONENT_VALUE_ID = cv.COMPONENT_VALUE_ID
          and td.SOR_TABLE_NAME = i_tabname
          and fd.field_name <> 'xqo_cust_num';   
   
  DECLARE CONTINUE HANDLER for not_found
      SET at_end = 1; 

  --
  -- get the minimum lowerbound value of the first (oldest)  non CF partition 
  --
  SET v_lowerbound =
        ( select cast( min(lowvalue) as varchar(36) )
          from syscat.datapartitions b
          where b.tabname = i_tabname
          and b.tabschema = i_schema 
          and b.datapartitionname not like '%CF%'
          and b.datapartitionname not like '%THE_END%' 
          and status != 'I' )
        ;

  --
  -- get the minimum upperbound value of the first (oldest) non CF partition 
  --
  SET v_upperbound =
        ( select cast( min(highvalue) as varchar(36) )
          from syscat.datapartitions b
          where b.tabname = i_tabname
          and b.tabschema = i_schema 
          and b.datapartitionname not like '%CF%'
          and b.datapartitionname not like '%THE_END%' 
          and status != 'I' )
        ;

  --
  -- get the name of a user temporary tablespace with a block size of 32k 
  -- this tablespace will house the global temporary table
  --
  SET v_user_tablespace =  
        ( select cast ( min(tbspace) as varchar(64) )
          from syscat.tablespaces t
          where t.datatype = 'U'
          and t.pagesize = 32768 )
      ;


  --
  -- create the name of the global temporary table
  --
  SET v_temp_table = i_tabname || '_GTT' ;


  --
  -- create the dynamic sql to create the gtt
  -- 
  SET v_dynStmt = 'DECLARE GLOBAL TEMPORARY TABLE ' || v_temp_table
                  || ' LIKE '  || i_schema || '.'  || i_tabname
                  || ' ON COMMIT PRESERVE ROWS NOT LOGGED IN ' 
                  || v_user_tablespace 
                ;

  call exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN ;
  END IF;


  SET v_dynStmt = 'INSERT INTO SESSION' || '.' || v_temp_table
                  || ' SELECT * FROM ' || i_schema || '.' || i_tabname
                  || ' as txn WHERE EXISTS '
                  || '( SELECT 1 FROM ' || i_schema || '.' || 'fcm_alert as alrt WHERE result_id = 7 ' ;
                  

  	open c_prep ;

    
    while ( at_end = 0 ) do
      FETCH c_prep into v_fieldname, v_alert_type;
      	IF ( at_end=0 ) THEN
      	    if (l_counter=0) THEN
      	    	set v_dynStmt = v_dynStmt||' AND ( ( alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
      		ELSE
      			set v_dynStmt = v_dynStmt||' OR ( alrt.alert_Value = txn.'||rtrim(v_fieldname)||'  and alrt.alert_type = '''||v_alert_type||''')';
            END IF;
      	END IF; 
      set l_counter=l_counter+1;
    end while;
    -- BAD makes assumption that the l_counter was > 0 after while loop
    set v_dynStmt = v_dynStmt||') ) AND rqo_proc_utc_datetime BETWEEN ' || v_lowerbound 
                  || ' AND ' || v_upperbound 
                ;

  call exec_sql_and_log(i_tabname, i_schema, v_partition, i_preview_only_ind, i_locktimeout_cnt, v_dynStmt, o_sqlcode );

  IF ( o_sqlcode != 0 ) THEN
     RETURN ;
  END IF;


  SET o_temp_table = v_temp_table;

END P1
@

CREATE PROCEDURE roll_partition   ( IN i_tabname VARCHAR(50),
                                    IN i_schema  VARCHAR(30)
                                    )
SPECIFIC ROLL_PARTITION
DYNAMIC RESULT SETS 0
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  ROLL_PARTITION
--
--
-- Description:
--     Top level entry point for the partitioning maintenance.
--
--
-- Called By:
--
--
-- Calls:
--
--
-- Notes:     
-- 
--
-- Usage:
--
--
-- History:    
--      2008/07/14   leushe   created. Added parameter to check run the Sanity Check or not    
--         					  '0'-- Perform Sanity Check Call
--         					  '1'-- Do not perform Sanity Check Call
--	                          Added parameter to keep confirmed frauds or not    
--         					  '1'-- keep confirmed frauds 
--         					  '0'-- Do not keep confirmed frauds
--      2009/08/24   leushe   Changed the proc to not return explicit parameters, but return 
--                            the sqlcode via a result set. this changed the signature of the proc
--                            This change was made so SAS could call the proc, as it doesn't handle explicit
--                            Output parameters
--      2009/09/03   leushe   Added support for the LOCKTIMEOUT_RETRY_COUNT Column on the partition_definition table    
--      2012/04/26   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the update_cf_partitions proc
--      2012/08/17   leushe   Added support for the DETACH_WAIT_SEC Column on the partition_definition table, passed to call
--                            to the detach_partition proc
--
--
------------------------------------------------------------------------

P1: BEGIN

  DECLARE v_rowCount        INTEGER   DEFAULT 0;
  DECLARE v_detached_table  VARCHAR(50);

  DECLARE SQLCODE INTEGER DEFAULT 0;
  DECLARE SQLSTATE CHAR(5) DEFAULT '00000' ;
  DECLARE v_sqlcode integer DEFAULT 0;
  DECLARE v_sqlstate varchar(50) DEFAULT '00000' ;
  DECLARE v_sqlmsg varchar(30000)  ;
  DECLARE l_preview_only_ind CHAR(1) DEFAULT '0';
  DECLARE l_chk_sanity_ind CHAR(1) DEFAULT '0';
  DECLARE l_keep_fraud_ind CHAR(1) DEFAULT '0';
  DECLARE l_partition_datatype CHAR(9) DEFAULT '0';
  DECLARE l_locktimeout_cnt INTEGER DEFAULT 3;
  DECLARE l_detach_wait_sec integer DEFAULT 30;
    
      /* Declare cursor to return result set to calling program */ 
        
        DECLARE c_result CURSOR WITH RETURN FOR
            SELECT   v_sqlcode as OUT_SQLCODE
                    ,v_sqlstate as OUT_SQLSTATE
            FROM    SYSIBM.SYSDUMMY1;
  
      /* Initialize variables */
        VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
        
        
  -- Z.  upcase all arguments 
  SET i_tabname = upper(i_tabname);
  SET i_schema = upper(i_schema);
  SET v_sqlstate = i_tabname ;
  
  --
  -- verify i_tabname is indeed a partitioned table
  --
  SET v_rowCount = ( select count(*) from fmx_partition_definition where table_name = i_tabname );

  IF ( v_rowCount != 1 ) THEN
    SET v_sqlcode = -1;
    SET v_sqlstate = '-1';
    SET v_sqlmsg = '';
    CALL audit_partition_action(i_tabname, i_schema, NULL, i_tabname||' is not partition eligible.',
                                '0', v_sqlcode, v_sqlstate, v_sqlmsg);
    SET v_sqlstate = i_tabname;                                
    OPEN c_result;

    RETURN -1;
  END IF;

  SELECT LOCKTIMEOUT_RETRY_COUNT,PREVIEW_ONLY_IND,SKIP_SANITY_CHECK_IND,KEEP_FRAUD_IND,PARTITION_DATATYPE,DETACH_WAIT_SEC 
  		INTO l_locktimeout_cnt,L_PREVIEW_ONLY_IND,L_CHK_SANITY_IND,L_KEEP_FRAUD_IND,L_PARTITION_DATATYPE,L_DETACH_WAIT_SEC
  FROM FMX_PARTITION_DEFINITION
  WHERE TABLE_NAME = I_TABNAME;



  IF ( l_chk_sanity_ind = '0' ) THEN

    -- 0. sanity check its ok to go on
    call sanity_check_partition(i_tabname, i_schema, l_preview_only_ind, l_keep_fraud_ind, v_sqlcode) ;

    IF ( v_sqlcode != 0 ) THEN    
      OPEN c_result;

      RETURN -1;
    END IF; 
  END IF; 

  call attach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_partition_datatype,  v_sqlcode);   
  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;

  END IF;

  
  IF ( l_keep_fraud_ind = '1' ) THEN
  --
  -- 1b. create a global temp table and copy eligible rows into from the base table
    call create_partition_temp_table(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_detached_table, v_sqlcode);

    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 
   
  -- 2. detach oldest
  call detach_partition(i_tabname, i_schema, l_preview_only_ind, l_locktimeout_cnt, l_detach_wait_sec, v_detached_table, v_sqlcode);

  IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
  END IF;


 
  IF ( l_keep_fraud_ind = '1' ) THEN
  -- 3. update_cf
    call update_cf_partition(i_tabname, i_schema, 'PART_CF', l_preview_only_ind,l_locktimeout_cnt, l_partition_datatype,l_detach_wait_sec, v_sqlcode);
   
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;
  END IF; 

   
  -- 4. empty oldest
  call truncate(v_detached_table, i_schema, l_preview_only_ind, l_locktimeout_cnt, v_sqlcode);
    IF ( v_sqlcode != 0 ) THEN
      OPEN c_result;

      RETURN -1;
    END IF;


  -- 5. insert current timestamp into definition table
  --    this value is used to ensure the code does not
  --    execute more frequently than it is supposed to
  IF ( l_preview_only_ind = '0' ) THEN
      update fmx_partition_definition 
         set last_run_timestamp = ( current timestamp )
         where table_name = i_tabname ;

  END IF;
  
  VALUES ( SQLCODE) INTO  v_sqlcode;
  COMMIT;
  OPEN c_result;
   
END P1
@



